@@ -153,24 +153,29 @@ class Fairness:
         ----------
         _fairness_widget() or _print_evaluate()
         """
+        #check if evaluate hasn't run, only run if haven't
         if self.evaluate_status == 0:
+            #to show progress bar
             eval_pbar = tqdm(total=100, desc='Evaluate performance', bar_format='{l_bar}{bar}')
             eval_pbar.update(1)
+            #execute performance metrics from PerformanceMetrics class
             self._compute_performance(n_threads=n_threads, seed = seed, eval_pbar=eval_pbar)
             eval_pbar.set_description('Evaluate fairness')
+            #execute fairness metrics from FairnessMetrics class
             self._compute_fairness(n_threads=n_threads, seed = seed, eval_pbar=eval_pbar)
-
+            #to determine fairness conclusion based on inputs
             self._fairness_conclusion()
+            #set status to 1 after evaluate has run
             self.evaluate_status = 1
             eval_pbar.set_description('Evaluate')
             eval_pbar.update(100 - eval_pbar.n)
             eval_pbar.close()
             print('', flush=True)
-        
+        #to trigger widget        
         if visualize == True:
             output = False
             self._fairness_widget()
-
+        #to trigger evaluate printout
         if output == True:
             self._print_evaluate()
 
@@ -193,25 +198,32 @@ class Fairness:
         out : dictionary
             Fairness threshold and conclusion for the chosen protected variable
         """
+        #for feature importance, when privileged metric values have been overwritten during leave-one-out analysis
         if "priv_m_v" in kwargs:
             priv_m_v = kwargs["priv_m_v"]
             value = kwargs["value"]
+        #else run as per input values
         else:
             priv_m_v = self.fair_metric_obj.result.get(protected_feature_name).get("fair_metric_values").get(self.fair_metric_name)[1]
             value = self.fair_metric_obj.result[protected_feature_name]["fair_metric_values"].get(self.fair_metric_name)[0]
         
+        #to handle different variations of threhold value provided e.g. float, decimals, integer
         fair_threshold = self._compute_fairness_metric_threshold(priv_m_v)
 
         out = {}
-
+        #append threshold value to result
         out['threshold'] = fair_threshold
 
+        #if metric used is ratio based, means it will either be more than 1 or less than 1. So set n = 1 to see the difference.
         if FairnessMetrics.map_fair_metric_to_group.get(self.fair_metric_name)[2] == 'ratio':
             n = 1
+        #if metric used is pairty based, means it will either be more than 0 or less than 0 So set n = 0 to see the difference.
         elif FairnessMetrics.map_fair_metric_to_group.get(self.fair_metric_name)[2] == 'parity':
             n = 0
 
+        #find absolute difference of fair values calculated after metric has been applied
         f_value = abs(value - n)
+        #determine whether input values are fair or unfair depending on metrics applied
         if f_value <= fair_threshold:
             out['fairness_conclusion'] = 'fair'
         else:
@@ -228,6 +240,7 @@ class Fairness:
             fair_conclusion and threshold for every protected variable
         """
         self.fair_conclusion = {}
+        #to append each fair conclusion for each protected variable into a single dictionary
         for i in  self.model_params[0].p_var:
             self.fair_conclusion[i] = self._fair_conclude(i)
             
@@ -246,7 +259,7 @@ class Fairness:
         fair_threshold : float
                 Fairness metric threshold
         """
-
+        #to handle different variations of threhold value provided e.g. float, decimals, integer
         if self.fair_threshold > 1:
             self.fair_threshold = floor(self.fair_threshold)
             if FairnessMetrics.map_fair_metric_to_group.get(self.fair_metric_name)[2] == 'ratio':
@@ -278,15 +291,17 @@ class Fairness:
         ----------
         All calculations from every performance metric
         """
+        #to initialize PerformanceMetrics and exceute all the perf metrics at one go
         self.perf_metric_obj = PerformanceMetrics(self)
         self.perf_metric_obj.execute_all_perf(n_threads=n_threads, seed = seed, eval_pbar=eval_pbar)
+        #bring status bar to full after all perf metrics have been ran
         eval_pbar.update(1)
-        
+        #if calibration_curve function has been run, then set status to True        
         if self.perf_metric_obj.result["calibration_curve"] is None:
             self.evaluate_status_cali = False
         else:
             self.evaluate_status_cali = True 
-
+        #if perf_dynamic function has been run, then set status to True
         if self.perf_metric_obj.result['perf_dynamic'] is None:
             self.evaluate_status_perf_dynamics = False
         else:
@@ -312,11 +327,14 @@ class Fairness:
         ----------
         All calculations from every fairness metric
         """
+        #to initialize FairnessMetrics and exceute all the fair metrics at one go
         self.fair_metric_obj = FairnessMetrics(self)
         self.fair_metric_obj.execute_all_fair(n_threads=n_threads, seed = seed, eval_pbar=eval_pbar)
+        #bring status bar to full after all fair metrics have been ran
         eval_pbar.update(1)
         for i in self.model_params[0].p_var:
             for j in self._use_case_metrics['fair']:
+                #if user provides fair metric value input value for each protected variable
                 if self.fairness_metric_value_input is not None :
                     if i in self.fairness_metric_value_input.keys(): 
                         if j in self.fairness_metric_value_input[i].keys(): 
@@ -345,9 +363,10 @@ class Fairness:
         ----------
         Prints messages for the status of evaluate and tradeoff and generates model artifact
         """
+        #check if evaluate hasn't run, only run if haven't
         if self.evaluate_status == 0:
             self.evaluate(visualize=False, output=False, n_threads=n_threads)
-
+        #printout
         print('{:40s}{:<10}'.format('Running evaluate','done'))
         print('{:5s}{:35s}{:<10}'.format('','performance measures','done'))
         print('{:5s}{:35s}{:<10}'.format('','bias detection','done'))
@@ -361,50 +380,63 @@ class Fairness:
             print('{:5s}{:35s}{:<10}'.format('','performance dynamics','done'))
         else:
             print('{:5s}{:35s}{:<10}'.format('','performance dynamics','skipped'))
-
+        #check if user wants to skip tradeoff, if yes tradeoff will not run, print skipped
         if self.tradeoff_status == -1:
             print('{:40s}{:<10}'.format('Running tradeoff','skipped'))
+        #check if tradeoff hasn't run and user does not want to skip, only run if haven't
         elif self.tradeoff_status == 0 and skip_tradeoff_flag==0:
             try :
                 self.tradeoff(output=False, n_threads=n_threads)
+                #if user wants to skip tradeoff, print skipped
                 if self.tradeoff_status == -1 :
                     print('{:40s}{:<10}'.format('Running tradeoff','skipped'))
+                #set status to 1 after evaluate has run
                 elif self.tradeoff_status == 1 :
                     print('{:40s}{:<10}'.format('Running tradeoff','done'))
             except :
                 print('{:40s}{:<10}'.format('Running tradeoff','skipped'))
+        #check if tradeoff hasn't run and user wants to skip, print skipped
         elif self.tradeoff_status == 0 and skip_tradeoff_flag==1:    
             self.tradeoff_status = -1 
             print('{:40s}{:<10}'.format('Running tradeoff','skipped'))
         else: 
             print('{:40s}{:<10}'.format('Running tradeoff','done'))
-
-        if self.feature_imp_status == -1:
+        #check if user wants to skip feature_importance, if yes feature_importance will not run, print skipped
+        if self.feature_imp_status_corr:
+            print('{:40s}{:<10}'.format('Running feature importance','done'))            
+        elif self.feature_imp_status == -1:
             print('{:40s}{:<10}'.format('Running feature importance','skipped'))
+        #check if feature_importance hasn't run and user does not want to skip, only run if haven't
         elif self.feature_imp_status == 0 and skip_feature_imp_flag ==0:
             try :
                 self.feature_importance(output=False, n_threads=n_threads)
                 if self.feature_imp_status == 1:
                     print('{:40s}{:<10}'.format('Running feature importance','done'))
+                elif self.feature_imp_status_corr:
+                    print('{:40s}{:<10}'.format('Running feature importance','done'))
                 else:
                     print('{:40s}{:<10}'.format('Running feature importance','skipped'))
             except:
                 print('{:40s}{:<10}'.format('Running feature importance','skipped'))
+        #check if feature_importance hasn't run and user wants to skip, print skipped
         elif self.feature_imp_status == 0 and skip_feature_imp_flag ==1:
             self.feature_imp_status = -1
             print('{:40s}{:<10}'.format('Running feature importance','skipped'))            
         else:
             print('{:40s}{:<10}'.format('Running feature importance','done'))
+        #check if feature_importance_loo has ran, if not print skipped
 
         if self.feature_imp_status_loo:
             print('{:5s}{:35s}{:<10}'.format('','leave-one-out analysis','done'))
         else:
             print('{:5s}{:35s}{:<10}'.format('','leave-one-out analysis','skipped'))
+        #check if feature_importance_corr has ran, if not print skipped
         if self.feature_imp_status_corr:
             print('{:5s}{:35s}{:<10}'.format('','correlation analysis','done'))
         else:
             print('{:5s}{:35s}{:<10}'.format('','correlation analysis','skipped'))
 
+        #run function to generate json model artifact file after all API functions have ran 
         self._generate_model_artifact()
 
 
@@ -421,34 +453,37 @@ class Fairness:
         n_threads : int, default=1
                 Number of currently active threads of a job
         """
-        # if FairnessMetrics.map_fair_metric_to_group.get(self.fair_metric_name)[3] == False:
-        #     self.tradeoff_status = -1
-        #     print("Tradeoff has been skipped due to fair_metric_name")
-        #     return
+        #if y_prob is None, skip tradeoff
         if self.model_params[0].y_prob is None:
             self.tradeoff_status = -1
             print("Tradeoff has been skipped due to y_prob")
-            
+        #if user wants to skip tradeoff, return None            
         if self.tradeoff_status == -1:
             return
-
+        #check if tradeoff hasn't run, only run if haven't
         elif self.tradeoff_status == 0:
             n_threads = check_multiprocessing(n_threads)
+            #to show progress bar
             tdff_pbar = tqdm(total=100, desc='Tradeoff', bar_format='{l_bar}{bar}')
             tdff_pbar.update(5)
             sys.stdout.flush()
+            #initialize tradeoff
             self.tradeoff_obj = TradeoffRate(self)
             
             tdff_pbar.update(10)
+            #run tradeoff
             self.tradeoff_obj.compute_tradeoff(n_threads, tdff_pbar)
             tdff_pbar.update(100 - tdff_pbar.n)
             tdff_pbar.close()
             print('', flush=True)
-            if self.tradeoff_obj.result == None:
+            #if after running tradoeff, result is None, print skipped
+            if self.tradeoff_obj.result == {}:
                 print(self.tradeoff_obj.msg)
                 self.tradeoff_status = -1
             else:
+                #set status to 1 after tradeoff has ran
                 self.tradeoff_status = 1
+        #if tradeoff has already ran once, just print result
         if output and self.tradeoff_status == 1:
             self._print_tradeoff()
 
@@ -477,14 +512,15 @@ class Fairness:
 
         self._print_feature_importance()
         """
+        #if feature_imp_status_corr hasn't run
         if self.feature_imp_status_corr == False:
             self._compute_correlation()  
-            
+        #if user wants to skip feature_importance, return None            
         if self.feature_imp_status == -1:
             self.feature_imp_values = None
             return    
             
-        #to check callable functions
+        #check if feature_importance hasn't run, only run if haven't
         if self.feature_imp_status == 0:
 
             for k in self.model_params:
@@ -494,18 +530,21 @@ class Fairness:
                 x_test = k.x_test
                 train_op_name = k.train_op_name
                 predict_op_name = k.predict_op_name
+                # if model_object is not provided, skip feature_importance
                 if model_object is None:
                     self.feature_imp_status = -1
                     print("Feature importance has been skipped due to model_object")
                     return
                 else :
                     for var_name in [train_op_name, predict_op_name]:
+                        #to check callable functions
                         try:
                             callable(getattr(model_object, var_name)) 
                         except:
                             self.feature_imp_status = -1
                             print("Feature importance has been skipped due to train_op_name/predict_op_name error")
                             return
+            #to show progress bar
             fimp_pbar = tqdm(total=100, desc='Feature importance', bar_format='{l_bar}{bar}')
             fimp_pbar.update(1)
             self.feature_imp_values = {}
@@ -513,10 +552,11 @@ class Fairness:
                 self.feature_imp_values[h] = {}
             fimp_pbar.update(1)
 
-        # if evaluate_status = 0, call evaluate() first
+        #if evaluate_status = 0, run evaluate() first
             if self.evaluate_status == 0:
                 self.evaluate(output=False)
                 
+            #if user wants to skip feature_importance, return None
             if self.feature_imp_status == -1:
                 self.feature_imp_values = None
                 return        
@@ -550,13 +590,14 @@ class Fairness:
                             for removed_pvar, values in thread.result().items():
                                 for pvar, v in values.items():
                                     self.feature_imp_values[pvar][removed_pvar] = v
-
+            #change flag after feature_importance has finished running
             self.feature_imp_status_loo = True
             self.feature_imp_status = 1
             fimp_pbar.update(2)
             fimp_pbar.update(100.0-fimp_pbar.n)
             fimp_pbar.close()
             print('', flush=True)
+        #if feature_importance has already ran once, just print result
         if output == True:
             self._print_feature_importance()
 
@@ -592,9 +633,7 @@ class Fairness:
 
         # loop through model_params
         for k in range(len(use_case_object.model_params)):
-            # if model_param >1, then
-            # check if model object is a modelwrapper? ##deepcopy will not work for model wrapper object
-            ## for uplift model type --> two model container --> need to train two models
+           	## for uplift model type --> two model container --> need to train two models
             ## when model param len =2, then it is uplift model
             p_var = use_case_object.model_params[k].p_var
             x_train = use_case_object.model_params[k].x_train
@@ -607,40 +646,47 @@ class Fairness:
             neg_label = use_case_object.model_params[k].neg_label
             train_op = getattr(model_object, use_case_object.model_params[k].train_op_name)
             predict_op = getattr(model_object, use_case_object.model_params[k].predict_op_name)
+            #show progress bar
             fimp_pbar.update(round(worker_progress*0.9/len(use_case_object.model_params), 2))
 
             try:
+                #check if x_train is a dataframe
                 if isinstance(x_train, pd.DataFrame):
+                    #drop protected variable and train model                 
                     pre_loo_model_obj = train_op(x_train.drop(columns=[p_variable]), y_train)  # train_op_name is string, need to use getattr[] to get the attribute?
                 else :
                     pre_loo_model_obj = train_op(x_train, y_train, p_variable) # train_op to handle drop column i inside train_op
                     # Predict and compute performance Metrics (PerformanceMetrics.result.balanced_acc)
             except:
+                #else print skipped and return None
                 print("LOO analysis is skipped for [", p_variable, "] due to x_train/y_train error")
                 use_case_object.feature_imp_status = -1 
                 return None
 
             try:
+                #check if x_test is a dataframe
                 if isinstance(x_test, pd.DataFrame):
+                    #drop protected variable and predict  
                     pre_y_pred_new = np.array(predict_op(x_test.drop(columns=[p_variable])))
                 else :
                     pre_y_pred_new = predict_op(x_train, y_train, p_variable) # train_op to handle drop column i inside train_op
-                    # Predict and compute performance Metrics (PerformanceMetrics.result.balanced_acc)
             except:
+                #else print skipped and return None
                 print("LOO analysis is skipped for [", p_variable, "] due to x_test/y_test error")
                 use_case_object.feature_imp_status = -1 
                 return None
             
             fimp_pbar.update(round(worker_progress*0.02, 2))
             pre_y_pred_new = predict_op(x_test.drop(columns=[p_variable]))
+            #to ensure labels and datatype for predicted values are correct before running metrics
             if len(pre_y_pred_new.shape) == 1 and pre_y_pred_new.dtype.kind in ['i','O','U']:
                 pre_y_pred_new, pos_label2 = check_label(pre_y_pred_new, pos_label, neg_label)
             else:
                 pre_y_pred_new = pre_y_pred_new.astype(np.float64)
             y_pred_new.append(pre_y_pred_new)
-        # loo_values to store values for each protected variable for each protected variable that is being dropped
-        # loo_values = {}
+        #run performance and fairness evaluation only for primary performance and fair metric
         loo_perf_value = use_case_object.perf_metric_obj.translate_metric(use_case_object.perf_metric_name, y_pred_new=y_pred_new)
+        ##to find deltas (removed - baseline) for primary perf metric 
         deltas_perf = loo_perf_value - baseline_values[0]
 
         # to iterate through each protected variable for each protected variable that is being dropped
@@ -650,7 +696,7 @@ class Fairness:
             ## get loo_perf_value,loo_fair_values
             loo_fair_value, loo_priv_m_v = use_case_object.fair_metric_obj.translate_metric(use_case_object.fair_metric_name, y_pred_new=y_pred_new)[:2]
 
-            ##to find deltas (removed - baseline) for each protected variable in iteration
+            ##to find deltas (removed - baseline) for each protected variable in iteration for primary fair metric
             deltas_fair = loo_fair_value - baseline_values[1]
 
             ##fairness fair_conclusion
@@ -658,8 +704,10 @@ class Fairness:
             delta_conclusion = baseline_values[2] + " to " + loo_fairness_conclusion["fairness_conclusion"]
 
             ##suggestion
+            #if metric used is parity based, means it will either be more than 0 or less than 0. So set n = 0 to see the difference.
             if FairnessMetrics.map_fair_metric_to_group.get(use_case_object.fair_metric_name)[2] == 'parity':
                 n = 0
+            #if metric used is ratio based, means it will either be more than 1 or less than 1. So set n = 1 to see the difference.
             else:
                 n = 1
 
@@ -711,8 +759,10 @@ class Fairness:
         """
         Computes the top-20 correlation matrix inclusive of the protected variables
         """
-        #check if x_test is a DataFrame
         try :
+            if isinstance(self.model_params[0].x_test, str):
+                self.feature_imp_status_corr = False
+                return
             if isinstance(self.model_params[0].feature_imp, pd.DataFrame) and isinstance(self.model_params[0].x_test, pd.DataFrame):
                 #sort feature_imp dataframe by values (descending)
                 sorted_dataframe = self.model_params[0].feature_imp.sort_values(by=self.model_params[0].feature_imp.columns[1], ascending=False)
@@ -721,10 +771,12 @@ class Fairness:
                 p_var_cols = np.array(self.model_params[0].p_var)
                 feature_cols = [col for col in feature_cols if col not in p_var_cols]
                 feature_cols = feature_cols[:20-len(p_var_cols)]
-                #create final array
-                final_cols = np.append(feature_cols,p_var_cols)
-                #filter final_cols on x_test and apply corr()
-                df = self.model_params[0].x_test[final_cols].corr()
+                #feature_columns value from x_test
+                feature_columns = self.model_params[0].x_test[feature_cols]
+                #p_var_columns value from protected_features_cols
+                p_var_columns = self.model_params[0].x_test[p_var_cols]
+                #create final columns and apply corr()
+                df = pd.concat([feature_columns, p_var_columns], axis=1).corr()
                 self.correlation_output = {"feature_names":df.columns.values, "corr_values":df.values}
                 #return correlation_output as dataframe
                 self.feature_imp_status_corr = True
@@ -734,11 +786,12 @@ class Fairness:
                 p_var_cols = np.array(self.model_params[0].p_var)
                 feature_cols = [col for col in feature_cols if col not in p_var_cols]
                 feature_cols = feature_cols[:20-len(p_var_cols)]
-                #create final array
-                final_cols = np.append(feature_cols,p_var_cols)
-                #filter final_cols on x_test and apply corr()
-                df = self.model_params[0].x_test[final_cols]
-                df = df.corr()
+                #feature_columns value from x_test
+                feature_columns = self.model_params[0].x_test[feature_cols]
+                #p_var_columns value from protected_features_cols
+                p_var_columns = self.model_params[0].x_test[p_var_cols]
+                #create final columns and apply corr()
+                df = pd.concat([feature_columns, p_var_columns], axis=1).corr()
                 self.correlation_output = {"feature_names":df.columns.values, "corr_values":df.values}
                 self.feature_imp_status_corr = True
 
@@ -1003,27 +1056,28 @@ class Fairness:
         """
         Generates the JSON file to be saved locally at the end of compile()
         """
-        # aggregate the results into model artifact
+        #aggregate the results into model artifact
         print('{:40s}'.format('Generating model artifact'), end='')
         artifact = {}
 
         # Section 1 - fairness_init
-
+        #write results to fairness_init
         fairness_init = {}
         fairness_init["fair_metric_name_input"] = self.fair_metric_input
-        fairness_init["fair_metric_name"] = FairnessMetrics.map_fair_metric_to_group.get(self.fair_metric_name)[0] #self.fair_metric_name  # from usecase.fair_metric_name
-        fairness_init["perf_metric_name"] = PerformanceMetrics.map_perf_metric_to_group.get(self.perf_metric_name)[0] #self.perf_metric_name  # from usecase.perf_metric_name 
-        fairness_init["protected_features"] = self.model_params[0].p_var  # from model_params[0].p_var
+        fairness_init["fair_metric_name"] = FairnessMetrics.map_fair_metric_to_group.get(self.fair_metric_name)[0] 
+        fairness_init["perf_metric_name"] = PerformanceMetrics.map_perf_metric_to_group.get(self.perf_metric_name)[0] 
+        fairness_init["protected_features"] = self.model_params[0].p_var  
         if FairnessMetrics.map_fair_metric_to_group.get(self.fair_metric_name)[1] != "regression":
             fairness_init["fair_priority"] = self.fair_priority
             fairness_init["fair_concern"] = self.fair_concern
             fairness_init["fair_impact"] = self.fair_impact
         if self.model_params[0].model_type == "uplift" or self.model_params[0].model_type == "credit": 
-            fairness_init["special_params"] = self.spl_params  # num_applicants and base_default_rate for CS, treatment_cost, revenue and selection_threshold for CM
+            fairness_init["special_params"] = self.spl_params  #num_applicants and base_default_rate for creditscoring, treatment_cost, revenue and selection_threshold for customermarketing
         fairness_init["fair_threshold_input"] = self.fair_threshold_input
-        fairness_init["fair_neutral_tolerance"] = self.fair_neutral_tolerance  # usecase.fair_neutral_tolerance
-        model_type = self.model_params[0].model_type  # from model_params[0].model_type
-        artifact["fairness_init"] = fairness_init  # from model_param and other places
+        fairness_init["fair_neutral_tolerance"] = self.fair_neutral_tolerance  
+        model_type = self.model_params[0].model_type  
+        #add fairness_init results to artifact        
+        artifact["fairness_init"] = fairness_init  
         perf_result = deepcopy(self.perf_metric_obj.result)
         perf_vals_wth_metric_names = {}
         for key in self.perf_metric_obj.result["perf_metric_values"].keys():
@@ -1036,6 +1090,7 @@ class Fairness:
         # above part will only be tested when Credit Scoring and Customer Marketing classes can be run
         
         p_var = self.model_params[0].p_var
+        #write results to features_dict
         features_dict = {}
         for pvar in p_var:
             dic_h = {}
@@ -1049,7 +1104,7 @@ class Fairness:
             dic_t["feature_importance"] = None
             if self.feature_imp_status != -1:
                 dic_t["feature_importance"] = self.feature_imp_values.get(pvar)
-            #fair_metric_obj
+
             fair_vals_wth_metric_names = {}
             for key in self.fair_metric_obj.result.get(pvar)['fair_metric_values'].keys():
                 if key in FairnessMetrics.map_fair_metric_to_group.keys():                    
@@ -1060,13 +1115,10 @@ class Fairness:
             for k, v in fair_result['fair_metric_values'].items():
                 fair_result['fair_metric_values'][k] = [v[0], v[2]]
             features_dict[str(pvar)] = {**dic_h, **fair_result, **dic_t}
+        #add features_dict results to artifact
         artifact["features"] = features_dict
         print('done')
-        #model_name=''
-        # for m_param in self.model_params:
-        #     model_name += (m_param.model_name + ' ')
-        #model_name = model_name.replace(" ","_")
-        model_name = self.model_params[0].model_name +"_"
+        model_name = (self.model_params[0].model_name +"_").replace(" ","_")
         filename = "model_artifact_" + model_name + datetime.datetime.today().strftime('%Y%m%d_%H%M') + ".json"
         self.artifact = artifact
         artifactJson = json.dumps(artifact, cls=NpEncoder)
@@ -1327,7 +1379,7 @@ class Fairness:
                     filtered_data = pd.DataFrame(result_fairness[protected_feature]['fair_metric_values'])
     
                     if model_type != 'Uplift' and PerformanceMetrics.map_perf_metric_to_group.get(self.perf_metric_name)[1] != "regression":
-                        filtered_data.loc[0,'disparate_impact'] = filtered_data['disparate_impact'][0] - 1  
+                        filtered_data.loc[0,'disparate_impact'] = filtered_data['disparate_impact'][0] - 1 
                     metrics = list(filtered_data.columns)
                     values = filtered_data.loc[0].values
     
@@ -1491,30 +1543,35 @@ class Fairness:
         Checks if model type input is valid
         """
         for i in self.model_params :
+            #throw an error if model_type provided is not in _model_type_to_metric_lookup
             if i.model_type not in self._model_type_to_metric_lookup.keys():
                 self.err.push('value_error', var_name="model_type", given=str(i.model_type),
                               expected=list(self._model_type_to_metric_lookup.keys()),
                               function_name="_model_type_input")
-                
+                #print any exceptions occured                
                 self.err.pop()
   
         model_size = self._model_type_to_metric_lookup[self.model_params[0].model_type][2]
+        #check if model_size provided based in model_type provided is accepted as per _model_type_to_metric_lookup
         if model_size > len(self.model_params):
                 self.err.push('length_error', var_name="model_type", given=str(len(self.model_params)),
                               expected=str(model_size),
                               function_name="_model_type_input")
+                #print any exceptions occured
                 self.err.pop()
+        #check if model_size is -1. If it is only take first set of model_params values
         elif model_size == -1:
             self.model_params = self.model_params[:1]
         else:
              self.model_params = self.model_params[:model_size]       
                 
+        #check if model_type of first model_container is uplift, the model_name of second model_container should be clone. Otherwise, throw an exception            
         if self.model_params[0].model_type == 'uplift':
             if self.model_params[1].model_name != "clone" :
                 self.err.push('value_error', var_name="model_name", given=str(self.model_params[1].model_name),
                                   expected="clone",
                                   function_name="_model_type_input")
-        
+            #print any exceptions occured        
             self.err.pop()
 
     def _fairness_metric_value_input_check(self):
@@ -1523,11 +1580,13 @@ class Fairness:
         """
         if self.fairness_metric_value_input is not None: 
             for i in self.fairness_metric_value_input.keys() :
+                #if user provided keys are not in protected variables, ignore
                 if i not in self.model_params[0].p_var:
                     print("The fairness_metric_value_input is not provided properly, so it is ignored")
                     self.fairness_metric_value_input = None
                     break
                 for j in self.fairness_metric_value_input[i].keys():
+                    #if user provided fair metrics are not in fair metrics in use case class, ignore
                     if j not in self._use_case_metrics['fair']:
                         print("The fairness_metric_value_input is not provided properly, so it is ignored")
                         self.fairness_metric_value_input = None
@@ -1547,6 +1606,7 @@ class Fairness:
                 self.err.push('value_error', var_name="fair_metric_name", given=self.fair_metric_name,  expected=ratio_parity_metrics, function_name="check_fair_metric_name")
         except:
             pass
+        #print any exceptions occured
         self.err.pop()
 
     def check_perf_metric_name(self):
@@ -1563,6 +1623,7 @@ class Fairness:
                 self.err.push('value_error', var_name="perf_metric_name", given=self.perf_metric_name,  expected=perf_list, function_name="check_perf_metric_name")
         except:
             pass
+        #print any exceptions occured
         self.err.pop()
 
     def _fairness_tree(self, is_pos_label_favourable = True):
@@ -1665,6 +1726,106 @@ class Fairness:
         return self.fair_metric_name
 
 
+    def get_prob_calibration_results(self):
+        """
+        Gets the probability calibration results
+
+        Returns
+        ------------
+        a dictionary with below keys and values:
+            'prob_true': the ground truth values split into 10 bins from 0 to 1
+            'prob_pred': the mean predicted probability in each bin
+            'score': the brier loss score
+        """
+        if self.evaluate_status_cali == True:
+            return self.perf_metric_obj.result.get("calibration_curve")
+        else:
+            return None
+
+    def get_perf_metrics_results(self):
+        """
+        Gets the performance metrics results
+
+        Returns
+        ------------
+        a dictionary with keys as the metric name and values as the metric value together with confidence interval
+        """
+        if self.evaluate_status == 1:
+            return self.perf_metric_obj.result.get("perf_metric_values")
+        else: 
+            return None
+
+    def get_fair_metrics_results(self):
+        """
+        Gets the fair metrics results
+
+        Returns
+        ------------
+        a dictionary with keys as the metric name and values as the metric value together with confidence interval
+        """
+        if self.evaluate_status == 1:
+            result = {}
+            for p_var in self.fair_metric_obj.result.keys():
+                result[p_var] = self.fair_metric_obj.result[p_var]['fair_metric_values']
+            return result
+        else:
+            return None
+
+    def get_tradeoff_results(self):
+        """
+        Gets the tradeoff results
+
+        Returns
+        ------------
+        a dictionary with below keys and values:
+            protected variable name as key to split result values for each protected variable
+            'fair_metric_name': fairness metric name
+            'perf_metric_name': performance metric name
+            'fair': array of shape (n, n*) of fairness metric values
+            'perf': array of shape (n, n*) of performance metric values
+            'th_x': array of shape (n*, ) of thresholds on x axis
+            'th_y': array of shape (n*, ) of thresholds on y axis
+            'max_perf_point': maxiumn performance point on the grid
+            'max_perf_single_th': maxiumn performance point on the grid with single threshold
+            'max_perf_neutral_fair': maxiumn performance point on the grid with neutral fairness
+            *n is defined by tradeoff_threshold_bins in config
+        """
+        if self.tradeoff_status == 1:
+            return self.tradeoff_obj.result
+        else:
+            return None
+
+    def get_loo_results(self):
+        """
+        Gets the leave one out analysis results
+
+        Returns
+        ------------
+        a dictionary with below keys and values:
+            protected variable name as key to split fairness result on each protected variable
+            protected variable name as key to denote the removed protected variable
+            array values denote the performance metric value, fariness metric value, fairness conclusion and suggestion
+        """
+        if self.feature_imp_status_loo == True:
+            return self.feature_imp_values
+        else:
+            return None
+
+    def get_correlation_analysis_results(self):
+        """
+        Gets the correlation analysis results
+
+        Returns
+        ------------
+        a dictionary with below keys and values:
+            'feature_names': feature names for correlation analysis
+            'corr_values': correlation values according to feature names
+        """
+        if self.feature_imp_status_corr == True:
+            return self.correlation_output
+        else:
+            return None
+
 class NpEncoder(json.JSONEncoder):
     """
 
