@@ -42,6 +42,7 @@ class FairnessMetrics:
         'acquire_benefit': ('Benefit from Acquiring', 'uplift', 'parity', False, True)
     }
 
+    #to get cutomized metrics inherited from NewMetric class 
     for metric in newmetric.NewMetric.__subclasses__() :
         if metric.enable_flag ==True and metric.metric_type == "fair":
             map_fair_metric_to_group[metric.metric_name] =  (metric.metric_definition, metric.metric_group, metric.metric_parity_ratio, False, False)
@@ -89,7 +90,6 @@ class FairnessMetrics:
                 Contains all the performance & fairness metrics for each use case. 
                 {"fair ": ["fnr_parity", ...], "perf": ["balanced_accuracy, ..."]}
                 Dynamically assigned during initialisation by using the _metric_group_map in Fairness/Performance Metrics class and the _model_type_to_metric above.
-
         """
         self.map_fair_metric_to_method = {
             'disparate_impact': self._compute_disparate_impact,
@@ -117,6 +117,7 @@ class FairnessMetrics:
             'acquire_benefit': self._compute_benefit_from_acquiring
         }
 
+        #to get cutomized metrics inherited from NewMetric class
         for metric in NewMetric.__subclasses__() :
             if metric.enable_flag ==True and metric.metric_type == "fair":
                 self.map_fair_metric_to_method[metric.metric_name] =  metric.compute
@@ -135,7 +136,6 @@ class FairnessMetrics:
         self._use_case_metrics  = None
         self.use_case_object = use_case_object
 
-
     def execute_all_fair(self, n_threads, seed, eval_pbar):
         """
         Computes every fairness metric named inside the include_metrics list together with its associated confidence interval (dictionary), the privileged group metric value & the neutral position.
@@ -161,15 +161,15 @@ class FairnessMetrics:
                 - fairness metric value & corresponding confidence interval for all fairness metrics.
                 - feature distribution
         """
-        
         self.fair_metric_name = self.use_case_object.fair_metric_name
         self._use_case_metrics = self.use_case_object._use_case_metrics
         self.y_train = [model.y_train for model in self.use_case_object.model_params]
         self.p_var = [model.p_var for model in self.use_case_object.model_params]
         self.feature_mask = self.use_case_object.feature_mask
-
         self.curr_p_var = None
         self.result = {}
+
+        #initialize result structure
         for i in self.p_var[0]:
             self.result[i] = {}
             idx = self.feature_mask[i]
@@ -180,17 +180,21 @@ class FairnessMetrics:
             for j in self._use_case_metrics['fair']:
                 if j in self.map_fair_metric_to_method.keys():
                     self.result[i]["fair_metric_values"][j] = [] 
-                  
+        
+        #update progress bar by 10
         eval_pbar.update(10)
         n = len(self.use_case_object.model_params[0].y_true)
         n_threads = check_multiprocessing(n_threads)
 
+        #split k into k-1 times of random indexing compute and 1 time of original array compute
         if n_threads >= 1 and self.use_case_object.k > 1:
+            #prepare k-1 arrays of random indices
             indices = []
             np.random.seed(seed)
             for ind in range(self.use_case_object.k-1):
                 indices.append(np.random.choice(n, n, replace=True))
 
+            #split the indices based on number threads and put into indexes, the size each indexes is the number of times each thread need to run
             indexes = []
             for i in range(n_threads):
                 indexes.append([])
@@ -202,13 +206,16 @@ class FairnessMetrics:
             with concurrent.futures.ThreadPoolExecutor(max_workers = n_threads) as executor:
                 #iterate through protected variables to drop one by one as part of leave-on-out
                 for k in range(n_threads):
+                    #deepcopy will be skipped if n_threads is 1
                     if n_threads == 1:
                         metric_obj = self
                     else:
                         metric_obj = deepcopy(self)
+                    #submit each thread's work to thread pool
                     if len(indexes[k]) > 0:
                         threads.append(executor.submit(FairnessMetrics._execute_all_fair_map, metric_obj=metric_obj, index =indexes[k], eval_pbar=eval_pbar, worker_progress=worker_progress))
 
+                #retrive results from each thread
                 for thread in threads:
                     mp_result = thread.result()
                     for i in self.p_var[0]:
@@ -216,10 +223,13 @@ class FairnessMetrics:
                             if j in self.map_fair_metric_to_method.keys():
                                 self.result[i]["fair_metric_values"][j] += mp_result[i]["fair_metric_values"][j]
         else:
+            #if multithreading is not triggered, directly update the progress bar by 36
             eval_pbar.update(36)
 
+        #run 1 time of original array to compute fairness metrics
         FairnessMetrics._execute_all_fair_map(self, [np.arange(n)], eval_pbar, 1)
 
+        #generate the final fairness metrics values and their CI based on k times of computation
         for i in self.p_var[0]:
             for j in self._use_case_metrics['fair']:
                 if j in self.map_fair_metric_to_method.keys():
@@ -234,17 +244,17 @@ class FairnessMetrics:
         Maps each thread's work for execute_all_fair()
         Parameters
         ----------
-        metric_obj :
-        
-        index : 
-
+        metric_obj : FairnessMetrics object
+        index : array of shape (n,m)
         eval_pbar : tqdm object
                 Progress bar
-
-        worker_progress :             
+        worker_progress : int
+                Progress bar progress for each thread
         """
+        #get each iteration's progress in 2 decimals to update the progress bar
         prog = round(worker_progress/(len(index)), 2)
         for idx in index:
+            #prepare data
             metric_obj.y_true = [model.y_true[idx] for model in metric_obj.use_case_object.model_params]
             metric_obj.y_prob = [model.y_prob[idx] if model.y_prob is not None else None for model in metric_obj.use_case_object.model_params] 
             metric_obj.y_pred = [model.y_pred[idx] if model.y_pred is not None else None for model in metric_obj.use_case_object.model_params]
@@ -252,7 +262,8 @@ class FairnessMetrics:
             metric_obj.e_lift = metric_obj.use_case_object.e_lift[idx] if metric_obj.use_case_object.e_lift is not None else None
             metric_obj.pred_outcome = {k: v[idx] for k, v in metric_obj.use_case_object.pred_outcome.items()} if metric_obj.use_case_object.pred_outcome is not None else {None}
             metric_obj.feature_mask = {k: v[idx] for k, v in metric_obj.use_case_object.feature_mask.items()}
-                
+            
+            #compute fairness metrics for each p_var
             for i in metric_obj.p_var[0]:
                 metric_obj.curr_p_var = i
                 metric_obj.tp_p, metric_obj.fp_p, metric_obj.tn_p, metric_obj.fn_p, metric_obj.tp_u, metric_obj.fp_u, metric_obj.tn_u, metric_obj.fn_u  = metric_obj.use_case_object._get_confusion_matrix(metric_obj.y_true[0], metric_obj.y_pred[0], metric_obj.sample_weight[0], i, metric_obj.feature_mask)            
@@ -260,10 +271,10 @@ class FairnessMetrics:
                             metric_obj.result[i]["fair_metric_values"][j].append(metric_obj.map_fair_metric_to_method[j](obj=metric_obj))
 
             eval_pbar.update(prog)
+
         return metric_obj.result
 
     def translate_metric(self, metric_name, **kwargs ):
-
         """
         Computes the primary fairness metric value and its associate value for the privileged group, for the feature importance section.
         This function does not support rejection inference.
@@ -284,7 +295,6 @@ class FairnessMetrics:
                 - fairness metric value, corresponding confidence interval for chosen fairness metric.
                 - feature distribution
         """
-
         return self.map_fair_metric_to_method.get(metric_name)(**kwargs)
 
     def _compute_disparate_impact(self, **kwargs):
@@ -301,7 +311,6 @@ class FairnessMetrics:
         _compute_disparate_impact : tuple of floats
                 Fairness metric value and privileged group metric value
         """
-
         if 'y_pred_new' in kwargs:
             tp_p, fp_p, tn_p, fn_p, tp_u, fp_u, tn_u, fn_u = self.use_case_object._get_confusion_matrix(y_true=self.y_true[0], y_pred=kwargs['y_pred_new'][0], sample_weight = self.sample_weight[0], curr_p_var = self.curr_p_var, feature_mask = self.feature_mask)
         else:
@@ -312,7 +321,6 @@ class FairnessMetrics:
         
         return (pr_p/pr_u, pr_p)
 
-
     def _compute_demographic_parity(self, **kwargs):
         """
         Computes the difference in approval rate between the privileged and unprivileged groups
@@ -335,7 +343,6 @@ class FairnessMetrics:
 
         pr_p = (tp_p + fp_p) / (tp_p + fp_p + tn_p + fn_p)
         pr_u = (tp_u + fp_u) / (tp_u + fp_u + tn_u + fn_u)
-        
 
         return (pr_p - pr_u, pr_p)
 
@@ -358,7 +365,6 @@ class FairnessMetrics:
         else:
             tp_p, fp_p, tn_p, fn_p, tp_u, fp_u, tn_u, fn_u = self.tp_p, self.fp_p, self.tn_p, self.fn_p, self.tp_u, self.fp_u, self.tn_u, self.fn_u
 
-
         for_p = fn_p / (tn_p + fn_p)
         for_u = fn_u / (tn_u + fn_u)
 
@@ -777,7 +783,6 @@ class FairnessMetrics:
         
         return (((ppv_p - ppv_u) + (for_p - for_u)) / 2, (ppv_p - ppv_u) / 2)
     
-    
     def _compute_mi_independence(self, **kwargs) :
         """
         Compute Mutual Information independence
@@ -791,7 +796,6 @@ class FairnessMetrics:
         ----------
         _compute_mi_independence : tuple of floats
                 Fairness metric value and privileged group metric value
-
         """
         mask = self.feature_mask[self.curr_p_var]
         y_pred = self.y_pred[0]
@@ -807,7 +811,6 @@ class FairnessMetrics:
         
         return (mi_independence, None)
     
-    
     def _compute_mi_separation(self, **kwargs) :
         """
         Compute Mutual Information separation
@@ -822,7 +825,6 @@ class FairnessMetrics:
         _compute_mi_separation : tuple of floats
                 Fairness metric value and privileged group metric value
         """
-
         mask = self.feature_mask[self.curr_p_var]
         y_true = self.y_true[0]
         y_pred = self.y_pred[0]
@@ -839,7 +841,6 @@ class FairnessMetrics:
             
         return (mi_separation, None)
     
-
     def _compute_mi_sufficiency(self, **kwargs) :
         """
         Compute Mutual Information sufficiency
@@ -870,7 +871,6 @@ class FairnessMetrics:
             
         return (mi_sufficiency, None)
     
-
     def _get_entropy(self, df, columns) :
         """
         Compute the entropy
@@ -912,7 +912,6 @@ class FairnessMetrics:
         _compute_rejected_harm : tuple of floats
                 Fairness metric value and privileged group metric value
         """
-        
         if selection_threshold is None:
             selection_threshold = self.use_case_object.selection_threshold
 
@@ -928,7 +927,6 @@ class FairnessMetrics:
             e_lift = self.use_case_object._get_e_lift(y_pred_new=y_prob[1])
             pred_outcome = self.use_case_object._compute_pred_outcome(y_pred_new=y_prob)
 
-
         def _rej_harm(pred_outcome, selection_threshold, e_lift, mask_list) :
             bools = np.array([i > selection_threshold for i in e_lift])[mask_list]
             pRcT = pred_outcome['rej_treatment'][(mask_list)][(bools)]
@@ -961,7 +959,6 @@ class FairnessMetrics:
         _compute_rejected_harm : tuple of floats
                 Fairness metric value and privileged group metric value
         """
-       
         if selection_threshold is None:
             selection_threshold = self.use_case_object.selection_threshold
 
