@@ -4,12 +4,10 @@ from .fairness_metrics import FairnessMetrics
 from .modelrates import *
 from ..config.constants import Constants
 
-
 class TradeoffRate(object):
     """
     Class to compute the tradeoff between performance and fairness metrics across a range of threshold values.
     """
-
     def __init__(self, usecase_obj):
         """
         Parameters
@@ -68,7 +66,7 @@ class TradeoffRate(object):
         if self.metric_group == "uplift":
             if usecase_obj.spl_params['revenue'] is not None and usecase_obj.spl_params['treatment_cost'] is not None:                
                 if self.perf_metric_name not in ["expected_profit", "emp_lift"]:
-                    self.perf_metric_name = "expected_profit" #add spl_params check
+                    self.perf_metric_name = "expected_profit" 
             else :
                 self.perf_metric_name = "emp_lift"		
 
@@ -147,12 +145,12 @@ class TradeoffRate(object):
                     - max_perf_point
                     - max_perf_single_th
                     - max_perf_neutral_fair
-
         """
         self.result = {}
-        ### based on model type, compute the base rates accordingly
+        # based on model type, compute the base rates accordingly
         if self.metric_group == "uplift":
-            ### define meshgrid
+
+            # define meshgrid
             self.th_x = np.linspace(min(self.e_lift)*0.7, max(self.e_lift)*0.7, Constants().tradeoff_threshold_bins)
             self.th_y = np.linspace(min(self.e_lift)*0.7, max(self.e_lift)*0.7, Constants().tradeoff_threshold_bins)
             self.th_a, self.th_b = np.meshgrid(self.th_x, self.th_y, sparse=True)
@@ -160,11 +158,9 @@ class TradeoffRate(object):
             #access from the special params
             cost = self.spl_params["treatment_cost"]
             revenue = self.spl_params["revenue"]
-            if self.perf_metric_name == "expected_profit" and (cost==None or revenue==None):
-                self.msg = "Cost or revenue not provided."
-                return
             tdff_pbar.update(5)
             prog = round(80/(len(self.p_var)), 2)
+
             ## iterate through the protected_features
             for i in self.p_var:
                 self.curr_p_var = i
@@ -191,21 +187,23 @@ class TradeoffRate(object):
                 self.result[i]['max_perf_single_th'] = best_th1
                 self.result[i]['max_perf_neutral_fair'] = best_th3
                 tdff_pbar.update(prog)
+
         elif self.metric_group == "classification":
-            ## base rates for entire dataset to calculate the best performance metric grid used in _compute_max_perf()
+            # base rates for entire dataset to calculate the best performance metric grid used in _compute_max_perf()
             self.ths, self.tpr, self.fpr, self.ppv, self.forr, self.base_selection_rate, self.selection_rate = ModelRateClassify.compute_rates(self.y_true, self.y_prob, self.sample_weight)
 
-            ### define meshgrid
-            self.th_x = np.linspace(Constants().tradeoff_min_threshold, Constants().tradeoff_max_threshold, Constants().tradeoff_threshold_bins)
-            self.th_y = np.linspace(Constants().tradeoff_min_threshold, Constants().tradeoff_max_threshold, Constants().tradeoff_threshold_bins)
+            # define meshgrid
+            self.th_x = np.linspace(Constants().classify_min_threshold, Constants().classify_max_threshold, Constants().tradeoff_threshold_bins)
+            self.th_y = np.linspace(Constants().classify_min_threshold, Constants().classify_max_threshold, Constants().tradeoff_threshold_bins)
 
             self.th_a, self.th_b = np.meshgrid(self.th_x, self.th_y, sparse=True)
             tdff_pbar.update(5)
             prog = round(80/(len(self.p_var)), 2)
-            ## iterate through the protected_features
+
+            # iterate through the protected_features
             for i in self.p_var:
                 self.curr_p_var = i
-                ## initialising ModelRates object to access interpolated base rates
+                # initialising ModelRates object to access interpolated base rates
                 if self.sample_weight is None :
                     self.rates_a = ModelRateClassify(self.y_true[self.feature_mask[i]], self.y_prob[self.feature_mask[i]])
                     self.rates_b = ModelRateClassify(self.y_true[~self.feature_mask[i]], self.y_prob[~self.feature_mask[i]])                
@@ -213,19 +211,19 @@ class TradeoffRate(object):
                     self.rates_a = ModelRateClassify(self.y_true[self.feature_mask[i]], self.y_prob[self.feature_mask[i]], self.sample_weight[self.feature_mask[i]])
                     self.rates_b = ModelRateClassify(self.y_true[~self.feature_mask[i]], self.y_prob[~self.feature_mask[i]], self.sample_weight[~self.feature_mask[i]])
 
-                ## base rates for privileged and unprivileged groups to be used in classification based compute functions
+                # base rates for privileged and unprivileged groups to be used in classification based compute functions
                 self.tpr_a, self.tpr_b, self.fpr_a, self.fpr_b, self.selection_rate_a, self.selection_rate_b, self.forr_a, self.forr_b, self.ppv_a, self.ppv_b = self.rates_a.tpr(self.th_a), self.rates_b.tpr(self.th_b), self.rates_a.fpr(self.th_a), self.rates_b.fpr(self.th_b), self.rates_a.selection_rate(self.th_a), self.rates_b.selection_rate(self.th_b), self.rates_a.forr(self.th_a), self.rates_b.forr(self.th_b), self.rates_a.ppv(self.th_a), self.rates_b.ppv(self.th_b)
 
-                #### compute the metrics and create dictionary
+                # compute the metrics and create dictionary
                 fair_values = self.map_metric_to_method[self.fair_metric_name]()
 
-                ### add metric and threshold values to result dictionary
+                # add metric and threshold values to result dictionary
                 self.result[i]= {}
 
                 self.result[i]['fair_metric_name'] = self.fair_metric_name
                 self.result[i]['perf_metric_name'] = self.perf_metric_name
 
-                perf_values = self.map_metric_to_method[self.perf_metric_name]()                
+                perf_values = self.map_metric_to_method[self.perf_metric_name]()
 
                 self.result[i]['fair'] = fair_values
                 self.result[i]['perf'] = perf_values
@@ -242,6 +240,7 @@ class TradeoffRate(object):
             self.result = None
             self.msg = "Tradeoff has been skipped due to fair_metric_name"
             return
+
     @staticmethod
     def _compute_max_perf(self,perf_grid,fair_grid):
         """
@@ -264,13 +263,12 @@ class TradeoffRate(object):
         best_th3 : list
                 Best fairness-constrained performance metric and associated threshold
         """
-                
-        ## compute the best single performance metric and associated threshold
+        # compute the best single performance metric and associated threshold
         best_single_perf = np.max(np.fliplr(perf_grid).diagonal())
         best_single_th = self.th_x[np.argmax(np.fliplr(perf_grid).diagonal())]
         best_th1 = [best_single_th, best_single_th, best_single_perf]
 
-        ## compute the best single performance metric (split) and associated threshold
+        # compute the best single performance metric (split) and associated threshold
         idx = np.unravel_index(perf_grid.argmax(), perf_grid.shape)
         best_split_th_a, best_split_th_b = self.th_x[idx[1]], self.th_y[idx[0]]
         best_split_perf = perf_grid.max()
@@ -327,7 +325,7 @@ class TradeoffRate(object):
         f1 : list of lists of floats
                 Grid of f1 score values
         """
-        tpr_a, tpr_b = self.rates_a.tpr(self.th_a), self.rates_b.tpr(self.th_b)        
+        tpr_a, tpr_b = self.rates_a.tpr(self.th_a), self.rates_b.tpr(self.th_b)
         ppv_a, ppv_b = self.rates_a.ppv(self.th_a), self.rates_b.ppv(self.th_b)
         
         mask = self.feature_mask[self.curr_p_var]
@@ -335,7 +333,7 @@ class TradeoffRate(object):
         # Combine TPRs: P(R=1|Y=1) = P(R=1|Y=1,A=1)P(A=1|Y=1) + P(R=1|Y=1,A=0)P(A=0|Y=1)
         tpr = (tpr_a * np.mean(mask[self.y_true == 1]) +
                tpr_b * np.mean(~mask[self.y_true == 1]))
-        # Combine FPRs: P(R=1|Y=0) = P(R=1|Y=0,A=1)P(A=1|Y=0) + P(R=1|Y=0,A=0)P(A=0|Y=0)                        
+        # Combine FPRs: P(R=1|Y=0) = P(R=1|Y=0,A=1)P(A=1|Y=0) + P(R=1|Y=0,A=0)P(A=0|Y=0)
         ppv = (ppv_a * [np.mean(mask[self.y_prob > th]) for th in self.th_a.ravel()] ) + \
                 (ppv_b * [np.mean(~mask[self.y_prob >th]) for th in self.th_b.ravel()] )
         
@@ -532,7 +530,6 @@ class TradeoffRate(object):
         t_u = np.add(tr_u, tn_u)
         c_u = np.add(cr_u, cn_u)
 
-        
         treatment_response_rate = np.add((self.uplift_rates_a.emp_lift_tr(self.th_a))*t_p, self.uplift_rates_b.emp_lift_tr(self.th_b)*t_u)/(t_p + t_u)
         control_response_rate = np.add((self.uplift_rates_a.emp_lift_cn(self.th_a))*c_p, self.uplift_rates_b.emp_lift_cn(self.th_b)*c_u)/(c_p + c_u)
         
